// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:get_it/get_it.dart';
import 'package:injectable/injectable.dart';
import 'package:wyatt_continuous_deployment/src/core/injection/injectable.config.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/fastlane.dart';
import 'package:wyatt_continuous_deployment/src/domain/entities/flutter.dart';

final getIt = GetIt.instance;

@InjectableInit(
  initializerName: 'init',
  preferRelativeImports: false,
  asExtension: true,
)
Future<void> configureDependencies() => getIt.init().allReady();

@module
abstract class RegisterModule {
  @lazySingleton
  Flutter get flutter => Flutter();

  @lazySingleton
  Fastlane get fastlane => Fastlane();
}

class UsecaseInjectable extends Singleton {
  const UsecaseInjectable({
    super.as,
    super.env,
    super.dispose,
    super.scope,
    super.order,
  });
}

const usecase = UsecaseInjectable();

class RepositoryInjectable extends Singleton {
  const RepositoryInjectable({
    super.as,
    super.env,
    super.dispose,
    super.scope,
    super.order,
  });
}

class DataSourceInjectable extends Singleton {
  const DataSourceInjectable({
    super.as,
    super.env,
    super.dispose,
    super.scope,
    super.order,
  });
}
