// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

/// {@template flat_button_theme_extension}
/// Flat button theme extension that extends [ThemeExtension] and
/// implements copyWith, and lerp methods so you don't have to.
/// {@endtemplate}
class FlatButtonThemeExtension
    extends ThemeExtension<FlatButtonThemeExtension> {
  /// {@macro flat_button_theme_extension}
  const FlatButtonThemeExtension({
    this.disabledStyle,
    this.focusedStyle,
    this.hoveredStyle,
    this.normalStyle,
    this.tappedStyle,
  });

  /// Style of this button in disabled state
  final FlatButtonStyle? disabledStyle;

  /// Style of this button in focused state
  final FlatButtonStyle? focusedStyle;

  /// Style of this button in hovered state
  final FlatButtonStyle? hoveredStyle;

  /// Style of this button in normal state
  final FlatButtonStyle? normalStyle;

  /// Style of this button in tapped state
  final FlatButtonStyle? tappedStyle;

  @override
  ThemeExtension<FlatButtonThemeExtension> copyWith({
    FlatButtonStyle? disabledStyle,
    FlatButtonStyle? focusedStyle,
    FlatButtonStyle? hoveredStyle,
    FlatButtonStyle? normalStyle,
    FlatButtonStyle? tappedStyle,
  }) =>
      FlatButtonThemeExtension(
        disabledStyle: disabledStyle ?? this.disabledStyle,
        focusedStyle: focusedStyle ?? this.focusedStyle,
        hoveredStyle: hoveredStyle ?? this.hoveredStyle,
        normalStyle: normalStyle ?? this.normalStyle,
        tappedStyle: tappedStyle ?? this.tappedStyle,
      );

  @override
  ThemeExtension<FlatButtonThemeExtension> lerp(
    covariant ThemeExtension<FlatButtonThemeExtension>? other,
    double t,
  ) {
    if (other is! FlatButtonThemeExtension) {
      return this;
    }
    return FlatButtonThemeExtension(
      disabledStyle:
          FlatButtonStyle.lerp(disabledStyle, other.disabledStyle, t),
      focusedStyle: FlatButtonStyle.lerp(focusedStyle, other.focusedStyle, t),
      hoveredStyle: FlatButtonStyle.lerp(hoveredStyle, other.hoveredStyle, t),
      normalStyle: FlatButtonStyle.lerp(normalStyle, other.normalStyle, t),
      tappedStyle: FlatButtonStyle.lerp(tappedStyle, other.tappedStyle, t),
    );
  }
}
