// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/core/design_system/colors.dart';

/// {@template card_theme_extension_impl}
/// Card theme extension that implements Wyatt Theme.
/// {@endtemplate}
class CardThemeExtensionImpl extends CardThemeExtension {
  /// {@macro card_theme_extension_impl}
  const CardThemeExtensionImpl({
    super.radius,
    super.padding,
    super.backgroundColors,
    super.borderColors,
    super.stroke,
    super.shadow,
    super.minSize,
    super.maxSize,
    super.titleStyle,
    super.subtitleStyle,
    super.bodyStyle,
  });

  /// {@macro card_theme_extension_impl}
  ///
  /// You can pass a [ThemeData] to use its text styles.
  ///
  /// This is the default light theme.
  factory CardThemeExtensionImpl.light({ThemeData? theme}) {
    theme ??= ThemeData.light();
    return CardThemeExtensionImpl(
      radius: const BorderRadius.all(Radius.circular(12)),
      padding: const EdgeInsets.all(25),
      stroke: 1,
      backgroundColors: const MultiColor.single(WyattColors.light),
      borderColors: WyattColors.lightGradient,
      titleStyle: theme.textTheme.titleLarge?.copyWith(
        fontSize: 26,
        fontWeight: FontWeight.w600,
      ),
      subtitleStyle: theme.textTheme.titleMedium,
      bodyStyle: theme.textTheme.bodyMedium?.copyWith(
        height: 1.5,
      ),
      minSize: const Size(330, 0),
      maxSize: const Size(390, double.infinity),
      shadow: BoxShadow(
        color: WyattColors.black.withOpacity(0.05),
        blurRadius: 30,
        offset: const Offset(0, 5),
      ),
    );
  }

  /// {@macro card_theme_extension_impl}
  ///
  /// This is the default dark theme.
  factory CardThemeExtensionImpl.dark({ThemeData? theme}) {
    theme ??= ThemeData.dark();
    return CardThemeExtensionImpl(
      radius: const BorderRadius.all(Radius.circular(12)),
      padding: const EdgeInsets.all(25),
      stroke: 1,
      backgroundColors: MultiColor.single(WyattColors.light.withOpacity(0.04)),
      borderColors: WyattColors.grayGradient,
      titleStyle: theme.textTheme.titleLarge?.copyWith(
        fontSize: 26,
        fontWeight: FontWeight.w600,
      ),
      subtitleStyle: theme.textTheme.titleMedium,
      bodyStyle: theme.textTheme.bodyMedium?.copyWith(
        height: 1.5,
      ),
      minSize: const Size(330, 0),
      maxSize: const Size(390, double.infinity),
      shadow: BoxShadow(
        color: WyattColors.black.withOpacity(0.15),
        blurRadius: 30,
        offset: const Offset(0, 5),
      ),
    );
  }
}
